import os
import re
import subprocess

from rich.console import Console
from termcolor import colored

console = Console()


def validate_projects_folder_path(projects_folder_path):
    msg = None

    if not projects_folder_path:
        msg = colored(
            ("\n\nERROR: You must provide the path to your projects folder."),
            "red",
            attrs=["bold"],
        )
    elif not os.path.exists(projects_folder_path):
        msg = colored(
            (
                f"\n\nERROR: {projects_folder_path} does not exist. "
                "Please provide a valid path."
            ),
            "red",
            attrs=["bold"],
        )

    if msg:
        print(msg)
        exit(1)


def validate_project_name(project_path):
    project_name = os.path.basename(project_path)
    msg = None

    if not project_name:
        msg = colored(
            ("\n\nERROR: You must provide the name of the project."),
            "red",
            attrs=["bold"],
        )

    elif os.path.exists(project_path):
        msg = colored(
            (
                f"\n\nERROR: {project_path} already exists. "
                "Please choose another name or delete the existing folder."
            ),
            "red",
            attrs=["bold"],
        )
    elif not re.match(r"^[a-zA-Z0-9-_]+$", project_name):
        msg = colored(
            (
                "\n\nERROR: Invalid project name. "
                "Project name can only contain letters, numbers, dashes, and underscores."
            ),
            "red",
            attrs=["bold"],
        )

    if msg:
        print(msg)
        exit(1)


def validate_options(
    non_interactive,
    project_folder,
    project_name,
    virtual_env,
    git,
    pre_commit,
    ruff,
    autoenv,
    makefile,
    pip_tools,
    username,
):
    valid = True

    if pre_commit:
        if not git:
            msg = colored(
                (
                    "\nERROR: No point in installing pre-commit without git. "
                    "Please enable the git option if you want to use pre-commit"
                ),
                "red",
                attrs=["bold"],
            )
            print(msg)
            valid = False

        try:
            with console.status("[green]Checking if pre-commit is installed..."):
                subprocess.run(
                    ["pre-commit", "--version"],
                    check=True,
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.DEVNULL,
                )
        except Exception:
            msg = colored(
                (
                    "\nWARNING: pre-commit is not installed. Please make sure to install "
                    "it\n\tpipx install pre-commit or brew install pre-commit"
                ),
                "yellow",
                attrs=["bold"],
            )
            print(msg)

            msg = colored(
                ("Continuing without pre-commit. You can install it later.\n"),
                "blue",
                attrs=["bold"],
            )
            print(msg)
            valid = False

    if pip_tools and not virtual_env:
        msg = colored(
            (
                "\nERROR: You shouldn't use pip-tools without a virtual environment. "
                "Please enable the virtual environment option."
            ),
            "red",
            attrs=["bold"],
        )
        print(msg)
        valid = False

    if autoenv and not virtual_env:
        msg = colored(
            (
                "\nERROR: No point in setting autoenv without a virtual "
                "environment. Please enable the virtual environment option."
            ),
            "red",
            attrs=["bold"],
        )
        print(msg)
        valid = False

    if non_interactive and (not project_name or not project_folder):
        msg = colored(
            "\nERROR: Both project name and project folder must be provided when running "
            "in non-interactive mode.\n",
            "red",
            attrs=["bold"],
        )
        print(msg)
        valid = False

    return valid



import os
import subprocess

from unittest.mock import call, mock_open

import pytest

from ppieces.utils import commands


@pytest.fixture
def mock_subprocess_run(mocker):
    return mocker.patch("subprocess.run", autospec=True)


@pytest.fixture
def mock_os_chdir(mocker):
    return mocker.patch("os.chdir", autospec=True)


@pytest.fixture
def mock_os_makedirs(mocker):
    return mocker.patch("os.makedirs", autospec=True)


@pytest.fixture
def mock_shutil_rmtree(mocker):
    return mocker.patch("shutil.rmtree", autospec=True)


def test_install_precommit_hooks(mock_subprocess_run, mock_os_chdir):
    project_path = "/fake/project/path"
    commands.install_precommit_hooks(project_path)
    calls = [call(project_path), call(os.getcwd())]
    mock_os_chdir.assert_has_calls(calls)
    mock_subprocess_run.assert_called_with(
        ["pre-commit", "install"],
        check=True,
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
    )


def test_setup_autoenv(mock_subprocess_run):
    project_path = "/fake/project/path"
    commands.setup_autoenv(project_path)
    mock_subprocess_run.assert_called_with(
        [os.path.join(commands.SCRIPTS_DIR, "setup_autoenv.sh"), project_path],
        check=True,
    )


def test_pip_install_requirements(mock_subprocess_run):
    project_path = "/fake/project/path"
    requirements_file_path = "requirements.txt"
    commands.pip_install_requirements(project_path, requirements_file_path)
    mock_subprocess_run.assert_called_with(
        [
            f"{project_path}/venv/bin/pip",
            "install",
            "-r",
            os.path.join(project_path, requirements_file_path),
        ],
        check=True,
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
    )


def test_create_virtual_environment(mock_subprocess_run, mocker):
    mock_add_and_install_requirements = mocker.patch(
        "ppieces.utils.commands.add_and_install_requirements"
    )
    project_path = "/fake/project/path"
    commands.create_virtual_environment(project_path, False)
    calls = [
        call(["python", "-m", "venv", os.path.join(project_path, "venv")], check=True),
        call(
            [f"{project_path}/venv/bin/pip", "install", "--upgrade", "pip"],
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        ),
    ]
    mock_subprocess_run.assert_has_calls(calls)
    mock_add_and_install_requirements.assert_called_once_with(project_path, False)


def test_initialize_git_repository(mock_subprocess_run, mocker):
    mocker.patch("ppieces.utils.copy.copy_gitignore_file")
    mocker.patch("ppieces.utils.copy.copy_readme_file")
    mocker.patch("ppieces.utils.copy.copy_template_file")
    mocker.patch("builtins.open", mock_open())
    mocker.patch("os.rename")
    project_path = "/fake/project/path"
    username = "fakeuser"
    commands.initialize_git_repository(project_path, username)
    calls = [
        call(["git", "init", project_path], check=True, stdout=-3, stderr=-3),
    ]
    mock_subprocess_run.assert_has_calls(calls)


def test_initial_commit(mock_subprocess_run):
    project_path = "/fake/project/path"
    commands.initial_commit(project_path)
    calls = [
        call(
            ["git", "add", "."],
            cwd=project_path,
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        ),
        call(
            ["git", "commit", "-m", "initial commit"],
            cwd=project_path,
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        ),
    ]
    mock_subprocess_run.assert_has_calls(calls)


def test_create_project_directory(mock_os_makedirs):
    project_path = "/fake/project/path"
    commands.create_project_directory(project_path)
    mock_os_makedirs.assert_called_with(project_path)


def test_add_and_install_requirements_with_pip_tools(
    mock_subprocess_run, mocker, tmp_path
):
    project_path = tmp_path / "fake_project"
    project_path.mkdir()
    mocker.patch("ppieces.utils.copy.copy_pip_tools_requirements_files")
    commands.add_and_install_requirements(str(project_path), True)
    calls = [
        call(
            [f"{project_path}/venv/bin/pip", "install", "pip-tools"],
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        ),
        call(
            [
                f"{project_path}/venv/bin/pip-compile",
                os.path.join(project_path, "requirements", "development.in"),
                "--output-file",
                os.path.join(project_path, "requirements", "development.txt"),
            ],
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        ),
    ]
    mock_subprocess_run.assert_has_calls(calls)


def test_add_and_install_requirements_without_pip_tools(
    mock_subprocess_run, mocker, tmp_path
):
    project_path = tmp_path / "fake_project"
    project_path.mkdir()
    mocker.patch("ppieces.utils.copy.copy_requirements_file")
    commands.add_and_install_requirements(project_path, False)
    mock_subprocess_run.assert_called_once_with(
        [
            f"{project_path}/venv/bin/pip",
            "install",
            "-r",
            os.path.join(project_path, "requirements.txt"),
        ],
        check=True,
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
    )


def test_create_project_directory_existing_dir(mock_os_makedirs):
    project_path = "/fake/project/path"
    mock_os_makedirs.side_effect = FileExistsError
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        commands.create_project_directory(project_path)
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 2
    mock_os_makedirs.assert_called_with(project_path)


def test_delete_path(mock_shutil_rmtree, mocker):
    project_path = "/fake/project/path"
    mocker.patch("ppieces.utils.prompts.ask_user", return_value=True)
    mocker.patch("builtins.input", return_value="yes")
    assert commands.delete_path(project_path) is True
    mock_shutil_rmtree.assert_called_with(project_path)


def test_delete_path_negative(mock_shutil_rmtree, mocker):
    project_path = "/fake/project/path"
    mocker.patch("ppieces.utils.prompts.ask_user", return_value=False)
    mocker.patch("builtins.input", return_value="no")
    assert commands.delete_path(project_path) is None
    mock_shutil_rmtree.assert_not_called()



import os

import pytest

from ppieces.utils import prompts


@pytest.fixture
def mock_input(mocker):
    return mocker.patch("builtins.input", side_effect=[""])


@pytest.fixture
def mock_print(mocker):
    return mocker.patch("builtins.print")


@pytest.fixture
def mock_os_system(mocker):
    return mocker.patch("os.system")


def test_welcome(mock_os_system, mock_print):
    prompts.welcome()
    mock_os_system.assert_called_once_with("clear")
    assert mock_print.call_count > 0


def test_bye(mock_print):
    prompts.bye()
    mock_print.assert_called_once()


def test_ask_user_yes(mock_input, mock_print):
    mock_input.side_effect = ["y"]
    assert prompts.ask_user("Is this a test?") is True


def test_ask_user_no(mock_input, mock_print):
    mock_input.side_effect = ["n"]
    assert prompts.ask_user("Is this a test?") is False


def test_ask_user_default_yes(mock_input, mock_print):
    mock_input.side_effect = [""]
    assert prompts.ask_user("Is this a test?") is True


def test_ask_user_invalid_input(mock_input, mock_print):
    mock_input.side_effect = ["maybe", "y"]
    assert prompts.ask_user("Is this a test?") is True
    assert mock_print.call_count == 3


def test_get_project_path_default(mock_input, mocker):
    mock_input.side_effect = ["", "test_project"]
    mocker.patch("ppieces.utils.prompts.validate_projects_folder_path")
    mocker.patch("ppieces.utils.prompts.validate_project_name")
    expected_path = os.path.join(os.getenv("HOME"), "projects", "test_project")
    assert prompts.get_project_path() == expected_path


def test_get_project_path_custom(mock_input, mocker):
    custom_path = "/custom/projects"
    mock_input.side_effect = [custom_path, "test_project"]
    mocker.patch("ppieces.utils.prompts.validate_projects_folder_path")
    mocker.patch("ppieces.utils.prompts.validate_project_name")
    expected_path = os.path.join(custom_path, "test_project")
    assert prompts.get_project_path() == expected_path



import glob
import os
import shutil

from termcolor import colored

from ppieces.utils.constants import TEMPLATES_DIR


def copy_template_file(filename, project_path):
    shutil.copy(
        os.path.join(TEMPLATES_DIR, filename),
        os.path.join(project_path, filename),
    )
    msg = colored(
        (f"Created a default {filename} file in {project_path}"),
        "yellow",
        attrs=["bold"],
    )
    print(msg)


def copy_requirements_file(project_path):
    copy_template_file("requirements.txt", project_path)


def copy_makefile(project_path, pip_tools):
    copy_template_file("Makefile", project_path)

    if pip_tools:
        pip_tools_makefile_path = os.path.join(TEMPLATES_DIR, "pip-tools/Makefile")
        with open(pip_tools_makefile_path, "r") as f:
            pip_tools_makefile = f.read()
        with open(os.path.join(project_path, "Makefile"), "a") as f:
            f.write(pip_tools_makefile)

        msg = colored(
            (f"Extended Makefile with pip-tools commands in {project_path}"),
            "yellow",
            attrs=["bold"],
        )
        print(msg)

    # if pip_tools is False, we need to add the install-requirements command
    # to the Makefile since it is different if using pip-tools or not
    else:
        install_requirements_command = (
            "\ninstall-requirements:"
            "\n\t$(info Installing requirements...)"
            "\n\t@$(PIP) install -r requirements.txt"
        )
        with open(os.path.join(project_path, "Makefile"), "a") as f:
            f.write(install_requirements_command)


def copy_pip_tools_requirements_files(project_path):
    pip_tools_templates_folder_path = os.path.join(TEMPLATES_DIR, "pip-tools")
    requirements_folder_path = os.path.join(project_path, "requirements")
    os.makedirs(requirements_folder_path, exist_ok=False)

    pattern = f"{pip_tools_templates_folder_path}/requirements/*.in"
    for requirement_file in glob.glob(pattern):
        shutil.copy(requirement_file, requirements_folder_path)

    msg = colored(
        (f"Created a requirements folder for pip-tools in {project_path}"),
        "yellow",
        attrs=["bold"],
    )
    print(msg)


def copy_main_file(project_path):
    copy_template_file("main.py", project_path)


def copy_precommit_config(project_path):
    copy_template_file(".pre-commit-config.yaml", project_path)


def copy_ruff_config(project_path):
    copy_template_file(".ruff.toml", project_path)


def copy_gitignore_file(project_path):
    copy_template_file(".gitignore", project_path)


def copy_readme_file(project_path, username):
    copy_template_file("README-sample.md", project_path)
    project_name = os.path.basename(project_path)
    with open(os.path.join(project_path, "README-sample.md"), "r") as f:
        readme = f.read()
        readme = readme.replace("{{project_name}}", project_name)
        readme = readme.replace("{{username}}", username)
    with open(os.path.join(project_path, "README-sample.md"), "w") as f:
        f.write(readme)
    os.rename(
        os.path.join(project_path, "README-sample.md"),
        os.path.join(project_path, "README.md"),
    )



import pytest

from click.testing import CliRunner

from ppieces.main import main
from ppieces.utils.validation import validate_project_name, validate_projects_folder_path


@pytest.fixture
def runner():
    return CliRunner()


def test_missing_project_folder(runner):
    result = runner.invoke(
        main,
        [
            "--non-interactive",
        ],
    )
    assert result.exit_code == 2
    assert (
        "Both project name and project folder must be provided when running "
        "in non-interactive mode"
    ) in result.output


def test_missing_project_name(runner, tmp_path):
    fake_projects_folder = tmp_path / "fake/folder"
    fake_projects_folder.mkdir(parents=True, exist_ok=True)
    result = runner.invoke(
        main,
        [
            "--non-interactive",
            f"--project-folder={fake_projects_folder}",
        ],
    )
    assert result.exit_code == 2
    assert (
        "Both project name and project folder must be provided when running "
        "in non-interactive mode"
    ) in result.output


def test_pre_commit_without_git(runner, tmp_path):
    fake_projects_folder = tmp_path / "fake/folder"
    fake_projects_folder.mkdir(parents=True, exist_ok=True)
    result = runner.invoke(
        main,
        [
            "--non-interactive",
            f"--project-folder={fake_projects_folder}",
            "--project-name=fake_project",
            "--pre-commit",
        ],
    )
    assert result.exit_code == 2
    assert ("No point in installing pre-commit without git") in result.output


def test_pre_commit_wit_git(runner, mocker, tmp_path):
    mocker.patch("ppieces.utils.validation.subprocess.run", return_value=0)
    fake_projects_folder = tmp_path / "fake/folder"
    fake_projects_folder.mkdir(parents=True, exist_ok=True)
    result = runner.invoke(
        main,
        [
            "--non-interactive",
            f"--project-folder={fake_projects_folder}",
            "--project-name=fake_project",
            "--pre-commit",
            "--git",
        ],
    )
    assert result.exit_code == 0


def test_pre_commit_is_not_installed(runner, mocker, tmp_path):
    mocker.patch("ppieces.utils.validation.subprocess.run", side_effect=Exception)

    fake_projects_folder = tmp_path / "fake/folder"
    fake_projects_folder.mkdir(parents=True, exist_ok=True)
    result = runner.invoke(
        main,
        [
            "--non-interactive",
            f"--project-folder={fake_projects_folder}",
            "--project-name=fake_project",
            "--pre-commit",
            "--git",
        ],
    )
    assert result.exit_code == 2
    assert (
        "WARNING: pre-commit is not installed. Please make sure to install"
    ) in result.output
    assert ("Continuing without pre-commit. You can install it later.") in result.output


def test_pip_tools_without_virtual_env(runner, tmp_path):
    fake_projects_folder = tmp_path / "fake/folder"
    fake_projects_folder.mkdir(parents=True, exist_ok=True)
    result = runner.invoke(
        main,
        [
            "--non-interactive",
            f"--project-folder={fake_projects_folder}",
            "--project-name=fake_project",
            "--pip-tools",
        ],
    )
    assert result.exit_code == 2
    assert (
        "You shouldn't use pip-tools without a virtual environment. "
        "Please enable the virtual environment option."
    ) in result.output


def test_autoenv_without_virtual_env(runner, tmp_path):
    fake_projects_folder = tmp_path / "fake/folder"
    fake_projects_folder.mkdir(parents=True, exist_ok=True)
    result = runner.invoke(
        main,
        [
            "--non-interactive",
            f"--project-folder={fake_projects_folder}",
            "--project-name=fake_project",
            "--autoenv",
        ],
    )
    assert result.exit_code == 2
    assert (
        "No point in setting autoenv without a virtual environment. "
        "Please enable the virtual environment option."
    ) in result.output


def test_validate_projects_folder_path_missing(runner, mocker):
    with pytest.raises(SystemExit) as pytest_wrapped_exception:
        validate_projects_folder_path("")
    assert pytest_wrapped_exception.type == SystemExit
    assert pytest_wrapped_exception.value.code == 1


def test_validate_projects_folder_path_nonexistent(runner, mocker):
    with pytest.raises(SystemExit) as pytest_wrapped_exception:
        validate_projects_folder_path("/nonexistent/path")
    assert pytest_wrapped_exception.type == SystemExit
    assert pytest_wrapped_exception.value.code == 1


def test_validate_project_name_empty(runner, mocker):
    with pytest.raises(SystemExit) as pytest_wrapped_exception:
        validate_project_name("")
    assert pytest_wrapped_exception.type == SystemExit
    assert pytest_wrapped_exception.value.code == 1


def test_validate_project_name_existing(runner, mocker, tmp_path):
    existing_project_path = tmp_path / "existing_project"
    existing_project_path.mkdir()
    with pytest.raises(SystemExit) as pytest_wrapped_exception:
        validate_project_name(str(existing_project_path))
    assert pytest_wrapped_exception.type == SystemExit
    assert pytest_wrapped_exception.value.code == 1


def test_validate_project_name_invalid_characters(runner, mocker):
    with pytest.raises(SystemExit) as pytest_wrapped_exception:
        validate_project_name("inval!d/pr#ject/nam^")
    assert pytest_wrapped_exception.type == SystemExit
    assert pytest_wrapped_exception.value.code == 1



import pytest

from click.testing import CliRunner

from ppieces.main import main
from ppieces.version import __version__


@pytest.fixture
def runner():
    return CliRunner()


def test_ctrl_c(runner, mocker, tmp_path):
    fake_projects_folder = tmp_path / "fake/folder"
    fake_projects_folder.mkdir(parents=True, exist_ok=True)
    fake_project = fake_projects_folder / "fake_project"
    mocker.patch("ppieces.utils.flows.setup_project", return_value=None)
    mocker.patch("ppieces.utils.flows.finalize_project", return_value=None)
    mocker.patch(
        "builtins.input",
        side_effect=[
            fake_projects_folder,
            fake_project,
            KeyboardInterrupt,
            "y",
        ],
    )
    result = runner.invoke(main)
    print(result.output)
    print(result.exception)
    assert result.exit_code == 0
    assert "Aborting..." in result.output


def test_cli_version(runner):
    result = runner.invoke(main, ["--version"])
    assert __version__ in result.output


def test_cli_help(runner):
    result = runner.invoke(main, ["--help"])
    assert "Usage: main [OPTIONS]" in result.output
    assert "--help" in result.output


def test_cli_non_interactive_with_piptools(runner, mocker, tmp_path):
    mocker.patch("ppieces.utils.commands.create_project_directory", return_value=None)
    mocker.patch("ppieces.utils.flows.setup_project", return_value=None)
    mocker.patch("ppieces.utils.flows.finalize_project", return_value=None)
    mocker.patch("ppieces.utils.commands.initial_commit", return_value=None)
    fake_projects_folder = tmp_path / "fake/folder"
    fake_projects_folder.mkdir(parents=True, exist_ok=True)
    fake_project = fake_projects_folder / "fake_project"

    result = runner.invoke(
        main,
        [
            "--non-interactive",
            f"--project-folder={fake_projects_folder}",
            "--project-name=fake_project",
            "--virtual-env",
            "--git",
            "--pre-commit",
            "--ruff",
            "--autoenv",
            "--makefile",
            "--pip-tools",
            "--username=fake_user",
        ],
    )
    print(result.output)
    print(result.exception)
    assert result.exit_code == 0

    # check for the directory and the files that should be created
    assert (fake_project).exists()
    assert (fake_project / ".git").exists()
    assert (fake_project / ".gitignore").exists()
    assert (fake_project / ".pre-commit-config.yaml").exists()
    assert (fake_project / ".ruff.toml").exists()
    assert (fake_project / ".autoenv.sh").exists()
    assert (fake_project / ".autoenv_leave.sh").exists()
    assert (fake_project / "requirements").exists()
    assert (fake_project / "requirements" / "development.txt").exists()
    assert (fake_project / "requirements" / "base.in").exists()
    assert (fake_project / "requirements" / "test.in").exists()
    assert (fake_project / "requirements" / "production.in").exists()
    assert (fake_project / "requirements" / "development.in").exists()
    assert (fake_project / "venv").exists()
    assert (fake_project / "README.md").exists()
    assert (fake_project / "Makefile").exists()
    assert (fake_project / "main.py").exists()


def test_cli_interactive(runner, mocker, tmp_path):
    fake_projects_folder = tmp_path / "fake/folder"
    fake_projects_folder.mkdir(parents=True, exist_ok=True)
    fake_project = fake_projects_folder / "fake_project"
    mocker.patch("ppieces.utils.flows.setup_project", return_value=None)
    mocker.patch("ppieces.utils.flows.finalize_project", return_value=None)
    mocker.patch(
        "builtins.input",
        side_effect=[
            fake_projects_folder,
            fake_project,
            "y",
            "y",
            "y",
            "y",
            "y",
            "y",
            "y",
            "y",
        ],
    )
    result = runner.invoke(main)
    assert result.exit_code == 0
    # check for the directory and the files that should be created
    assert (fake_project).exists()
    assert (fake_project / ".git").exists()
    assert (fake_project / ".gitignore").exists()
    assert (fake_project / ".pre-commit-config.yaml").exists()
    assert (fake_project / ".ruff.toml").exists()
    assert (fake_project / ".autoenv.sh").exists()
    assert (fake_project / ".autoenv_leave.sh").exists()
    assert (fake_project / "requirements").exists()
    assert (fake_project / "requirements" / "development.txt").exists()
    assert (fake_project / "requirements" / "base.in").exists()
    assert (fake_project / "requirements" / "test.in").exists()
    assert (fake_project / "requirements" / "production.in").exists()
    assert (fake_project / "requirements" / "development.in").exists()
    assert (fake_project / "venv").exists()
    assert (fake_project / "README.md").exists()
    assert (fake_project / "Makefile").exists()
    assert (fake_project / "main.py").exists()



import os

from unittest.mock import mock_open

import pytest

from ppieces.utils import copy


# Tests for the copy module will be added here.
@pytest.fixture
def mock_shutil_copy(mocker):
    return mocker.patch("shutil.copy", autospec=True)


@pytest.fixture
def mock_os_path_join(mocker):
    return mocker.patch("os.path.join", autospec=True)


@pytest.fixture
def mock_print(mocker):
    return mocker.patch("builtins.print", autospec=True)


def test_copy_template_file(mock_shutil_copy, mock_os_path_join, mock_print):
    filename = "sample.txt"
    project_path = "/fake/project/path"
    copy.copy_template_file(filename, project_path)
    mock_os_path_join.assert_called_with(project_path, filename)
    mock_shutil_copy.assert_called_with(
        mock_os_path_join.return_value, os.path.join(project_path, filename)
    )
    mock_print.assert_called()


def test_copy_requirements_file(mock_shutil_copy, mock_os_path_join, mock_print):
    project_path = "/fake/project/path"
    copy.copy_requirements_file(project_path)
    mock_os_path_join.assert_called_with(project_path, "requirements.txt")
    mock_shutil_copy.assert_called_with(
        mock_os_path_join.return_value, os.path.join(project_path, "requirements.txt")
    )
    mock_print.assert_called()


def test_copy_makefile(mock_shutil_copy, mock_os_path_join, mock_print):
    project_path = "/fake/project/path"
    pip_tools = False
    copy.copy_makefile(project_path, pip_tools)
    mock_os_path_join.assert_called_with(project_path, "Makefile")
    mock_shutil_copy.assert_called_with(
        mock_os_path_join.return_value, os.path.join(project_path, "Makefile")
    )
    mock_print.assert_called()


def test_copy_pip_tools_requirements_files(
    mock_shutil_copy, mock_os_path_join, mock_print, mocker
):
    project_path = "/fake/project/path"
    mocker.patch(
        "glob.glob",
        return_value=["/fake/templates/pip-tools/requirements/development.in"],
    )
    mocker.patch("os.makedirs", autospec=True)
    copy.copy_pip_tools_requirements_files(project_path)
    mock_os_path_join.assert_called_with(project_path, "requirements")
    mock_shutil_copy.assert_called()
    mock_print.assert_called()


def test_copy_main_file(mock_shutil_copy, mock_os_path_join, mock_print):
    project_path = "/fake/project/path"
    copy.copy_main_file(project_path)
    mock_os_path_join.assert_called_with(project_path, "main.py")
    mock_shutil_copy.assert_called_with(
        mock_os_path_join.return_value, os.path.join(project_path, "main.py")
    )
    mock_print.assert_called()


def test_copy_precommit_config(mock_shutil_copy, mock_os_path_join, mock_print):
    project_path = "/fake/project/path"
    copy.copy_precommit_config(project_path)
    mock_os_path_join.assert_called_with(project_path, ".pre-commit-config.yaml")
    mock_shutil_copy.assert_called_with(
        mock_os_path_join.return_value,
        os.path.join(project_path, ".pre-commit-config.yaml"),
    )
    mock_print.assert_called()


def test_copy_ruff_config(mock_shutil_copy, mock_os_path_join, mock_print):
    project_path = "/fake/project/path"
    copy.copy_ruff_config(project_path)
    mock_os_path_join.assert_called_with(project_path, ".ruff.toml")
    mock_shutil_copy.assert_called_with(
        mock_os_path_join.return_value, os.path.join(project_path, ".ruff.toml")
    )
    mock_print.assert_called()


def test_copy_gitignore_file(mock_shutil_copy, mock_os_path_join, mock_print):
    project_path = "/fake/project/path"
    copy.copy_gitignore_file(project_path)
    mock_os_path_join.assert_called_with(project_path, ".gitignore")
    mock_shutil_copy.assert_called_with(
        mock_os_path_join.return_value, os.path.join(project_path, ".gitignore")
    )
    mock_print.assert_called()


def test_copy_readme_file(mock_shutil_copy, mock_os_path_join, mock_print, mocker):
    project_path = "/fake/project/path"
    username = "fakeuser"
    mocker.patch(
        "builtins.open",
        mock_open(read_data="Project name: {{project_name}}\nUsername: {{username}}"),
    )
    mocker.patch("os.rename", autospec=True)
    copy.copy_readme_file(project_path, username)
    mock_os_path_join.assert_called_with(project_path, "README.md")
    mock_shutil_copy.assert_called_with(
        mock_os_path_join.return_value, os.path.join(project_path, "README-sample.md")
    )
    mock_print.assert_called()
    open.assert_called_with(os.path.join(project_path, "README-sample.md"), "w")
    os.rename.assert_called_with(
        os.path.join(project_path, "README-sample.md"),
        os.path.join(project_path, "README.md"),
    )



from ppieces.utils.commands import (
    create_virtual_environment,
    initial_commit,
    initialize_git_repository,
    install_precommit_hooks,
    setup_autoenv,
)
from ppieces.utils.copy import (
    copy_main_file,
    copy_makefile,
    copy_precommit_config,
    copy_ruff_config,
)


def setup_project(
    project_path,
    options,
    username,
):
    options_mapping = {
        "virtual_env": create_virtual_environment,
        "git": initialize_git_repository,
        "autoenv": setup_autoenv,
        "ruff": copy_ruff_config,
        "pre_commit": copy_precommit_config,
        "makefile": copy_makefile,
    }

    copy_main_file(project_path)

    # we need to pop the pip_tools option from the options dict
    # because it's not a function that we can call directly
    # pip_tools is a flag that we need to pass for the virtual_env and makefile options
    pip_tools = options.pop("pip_tools")

    for option, value in options.items():
        if value:
            if option == "git":
                options_mapping[option](project_path, username)
            elif option == "virtual_env" or option == "makefile":
                options_mapping[option](project_path, pip_tools)
            else:
                options_mapping[option](project_path)


def finalize_project(project_path, git, pre_commit):
    if pre_commit:
        install_precommit_hooks(project_path)

    # this should be the last step since we are making the initial commit
    # AFTER all the template files are copied to the new project folder
    if git:
        initial_commit(project_path)
